/*
 *  EchoCS by Davide Libenzi ( Simple TCP/IP Echo Client && Server )
 *  Copyright (C) 2003  Davide Libenzi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Davide Libenzi <davidel@xmailserver.org>
 *
 */

#if !defined(_DBLLISTS_H)
#define _DBLLISTS_H


struct list_head {
    struct list_head *__next;
    struct list_head *__prev;
};



#define DBL_LIST_HEAD_INIT(name)        { &(name), &(name) }

#define DBL_LIST_HEAD(name)             struct list_head name = DBL_LIST_HEAD_INIT(name)

#define DBL_INIT_LIST_HEAD(ptr) \
do { \
    (ptr)->__next = (ptr)->__prev = (ptr); \
} while (0)

#define DBL_LIST_ADD(newi, prev, next) \
do { \
    struct list_head *__prev = prev; \
    struct list_head *__next = next; \
    (__next)->__prev = newi; \
    (newi)->__next = __next; \
    (newi)->__prev = __prev; \
    (__prev)->__next = newi; \
} while (0)

#define DBL_LIST_ADDH(new, head)        DBL_LIST_ADD(new, head, (head)->__next)

#define DBL_LIST_ADDT(new, head)        DBL_LIST_ADD(new, (head)->__prev, head)

#define DBL_LIST_UNLINK(prev, next) \
do { \
    struct list_head *__prev = prev; \
    struct list_head *__next = next; \
    (next)->__prev = __prev; \
    (prev)->__next = __next; \
} while (0)

#define DBL_LIST_DEL(entry) \
do { \
    DBL_LIST_UNLINK((entry)->__prev, (entry)->__next); \
    DBL_INIT_LIST_HEAD(entry); \
} while (0)

#define DBL_LIST_EMTPY(head)            ((head)->__next == head)

#define DBL_LIST_SPLICE(list, head) \
do { \
    struct list_head *    first = (list)->__next; \
    if (first != list) { \
        struct list_head *	last = (list)->__prev; \
        struct list_head *	at = (head)->__next; \
        (first)->__prev = head; \
        (head)->__next = first; \
        (last)->__next = at; \
        (at)->__prev = last; \
    } \
} while (0)

#define DBL_HEAD_COPY(oldh, newh) \
do { \
    *(oldh) = (*newh); \
    (newh)->__next->__prev = (oldh); \
    (newh)->__prev->__next = (oldh); \
} while (0)

#define DBL_ITEM_IN_LIST(ptr)           (((ptr)->__prev != (ptr)) && ((ptr)->__next != (ptr)))

#define DBL_LIST_FIRST(head)            (((head)->__next != (head)) ? (head)->__next: NULL)

#define DBL_LIST_LAST(head)             (((head)->__prev != (head)) ? (head)->__prev: NULL)

#define DBL_LIST_ENTRY(ptr, type, member)   ((type *)((char *)(ptr) - (unsigned long)(&((type *)0)->member)))

#define DBL_LIST_FOR_EACH(pos, head)    for (pos = (head)->__next; pos != (head); pos = (pos)->__next)

#define DBL_END_OF_LIST(pos, head)      ((pos) == (head))

#define DBL_LIST_NEXT(pos, head)        (((pos)->__next != (head)) ? (pos)->__next: NULL)

#define DBL_LIST_PREV(pos, head)        (((pos)->__prev != (head)) ? (pos)->__prev: NULL)



#endif

